@@ -32,6 +32,8 @@
 #include <protobuf.h>
 #include <Zend/zend.h>
 
+#include "utf8.h"
+
 // -----------------------------------------------------------------------------
 // Native slot storage.
 // -----------------------------------------------------------------------------
@@ -56,7 +58,7 @@ size_t native_slot_size(upb_fieldtype_t type) {
 }
 
 bool native_slot_set(upb_fieldtype_t type, const zend_class_entry* klass,
-                     void* memory, zval* value) {
+                     void* memory, zval* value TSRMLS_DC) {
   switch (type) {
     case UPB_TYPE_STRING:
     case UPB_TYPE_BYTES: {
@@ -172,11 +174,31 @@ CASE(FLOAT,  DOUBLE, float)
 CASE(DOUBLE, DOUBLE, double)
 CASE(BOOL,   BOOL,   int8_t)
 CASE(INT32,  LONG,   int32_t)
-CASE(INT64,  LONG,   int64_t)
-CASE(UINT64, LONG,   uint64_t)
 CASE(ENUM,   LONG,   uint32_t)
 
 #undef CASE
+
+#if SIZEOF_LONG == 4
+#define CASE(upb_type, c_type)                        \
+    case UPB_TYPE_##upb_type: {                       \
+      SEPARATE_ZVAL_IF_NOT_REF(cache);                \
+      char buffer[MAX_LENGTH_OF_INT64];               \
+      sprintf(buffer, "%lld", DEREF(memory, c_type)); \
+      ZVAL_STRING(*cache, buffer, 1);                 \
+      return;                                         \
+    }
+#else
+#define CASE(upb_type, c_type)                        \
+    case UPB_TYPE_##upb_type: {                       \
+      SEPARATE_ZVAL_IF_NOT_REF(cache);                \
+      ZVAL_LONG(*cache, DEREF(memory, c_type));       \
+      return;                                         \
+    }
+#endif
+CASE(UINT64, uint64_t)
+CASE(INT64,  int64_t)
+#undef CASE
+
     case UPB_TYPE_UINT32: {
       // Prepend bit-1 for negative numbers, so that uint32 value will be
       // consistent on both 32-bit and 64-bit architectures.
@@ -210,7 +232,7 @@ CASE(ENUM,   LONG,   uint32_t)
       return;
     }
     default:
-      return EG(uninitialized_zval_ptr);
+      return;
   }
 }
 
@@ -245,7 +267,7 @@ void native_slot_get_default(upb_fieldtype_t type, zval** cache TSRMLS_DC) {
       return;
     }
     default:
-      return EG(uninitialized_zval_ptr);
+      return;
   }
 }
 
@@ -295,7 +317,7 @@ const upb_fielddef* map_entry_value(const upb_msgdef* msgdef) {
   return value_field;
 }
 
-const zend_class_entry* field_type_class(const upb_fielddef* field) {
+const zend_class_entry* field_type_class(const upb_fielddef* field TSRMLS_DC) {
   if (upb_fielddef_type(field) == UPB_TYPE_MESSAGE) {
     zval* desc_php = get_def_obj(upb_fielddef_subdef(field));
     Descriptor* desc = zend_object_store_get_object(desc_php TSRMLS_CC);
@@ -305,6 +327,7 @@ const zend_class_entry* field_type_class(const upb_fielddef* field) {
     EnumDescriptor* desc = zend_object_store_get_object(desc_php TSRMLS_CC);
     return desc->klass;
   }
+  return NULL;
 }
 
 // -----------------------------------------------------------------------------
@@ -435,7 +458,8 @@ void free_layout(MessageLayout* layout) {
   FREE(layout);
 }
 
-void layout_init(MessageLayout* layout, void* storage, zval** properties_table) {
+void layout_init(MessageLayout* layout, void* storage,
+                 zval** properties_table TSRMLS_DC) {
   int i;
   upb_msg_field_iter it;
   for (upb_msg_field_begin(&it, layout->msgdef), i = 0; !upb_msg_field_done(&it);
@@ -451,11 +475,12 @@ void layout_init(MessageLayout* layout, void* storage, zval** properties_table)
       *oneof_case = ONEOF_CASE_NONE;
     } else if (is_map_field(field)) {
       zval_ptr_dtor(property_ptr);
-      map_field_create_with_type(map_field_type, field, property_ptr);
+      map_field_create_with_type(map_field_type, field, property_ptr TSRMLS_CC);
       DEREF(memory, zval**) = property_ptr;
     } else if (upb_fielddef_label(field) == UPB_LABEL_REPEATED) {
       zval_ptr_dtor(property_ptr);
-      repeated_field_create_with_type(repeated_field_type, field, property_ptr);
+      repeated_field_create_with_type(repeated_field_type, field,
+                                      property_ptr TSRMLS_CC);
       DEREF(memory, zval**) = property_ptr;
     } else {
       native_slot_init(upb_fielddef_type(field), memory, property_ptr);
@@ -501,8 +526,8 @@ zval* layout_get(MessageLayout* layout, const void* storage,
   }
 }
 
-void layout_set(MessageLayout* layout, MessageHeader* header, const upb_fielddef* field,
-                zval* val) {
+void layout_set(MessageLayout* layout, MessageHeader* header,
+		const upb_fielddef* field, zval* val TSRMLS_DC) {
   void* storage = message_data(header);
   void* memory = slot_memory(layout, storage, field);
   uint32_t* oneof_case = slot_oneof_case(layout, storage, field);
@@ -515,7 +540,7 @@ void layout_set(MessageLayout* layout, MessageHeader* header, const upb_fielddef
     // zval in properties table first.
     switch (type) {
       case UPB_TYPE_MESSAGE: {
-        upb_msgdef* msg = upb_fielddef_msgsubdef(field);
+        const upb_msgdef* msg = upb_fielddef_msgsubdef(field);
         zval* desc_php = get_def_obj(msg);
         Descriptor* desc = zend_object_store_get_object(desc_php TSRMLS_CC);
         ce = desc->klass;
@@ -535,7 +560,7 @@ void layout_set(MessageLayout* layout, MessageHeader* header, const upb_fielddef
         break;
     }
 
-    native_slot_set(type, ce, memory, val);
+    native_slot_set(type, ce, memory, val TSRMLS_CC);
     *oneof_case = upb_fielddef_number(field);
   } else if (upb_fielddef_label(field) == UPB_LABEL_REPEATED) {
     // Works for both repeated and map fields
@@ -549,11 +574,11 @@ void layout_set(MessageLayout* layout, MessageHeader* header, const upb_fielddef
     upb_fieldtype_t type = upb_fielddef_type(field);
     zend_class_entry *ce = NULL;
     if (type == UPB_TYPE_MESSAGE) {
-      upb_msgdef* msg = upb_fielddef_msgsubdef(field);
+      const upb_msgdef* msg = upb_fielddef_msgsubdef(field);
       zval* desc_php = get_def_obj(msg);
       Descriptor* desc = zend_object_store_get_object(desc_php TSRMLS_CC);
       ce = desc->klass;
     }
-    native_slot_set(type, ce, value_memory(field, memory), val);
+    native_slot_set(type, ce, value_memory(field, memory), val TSRMLS_CC);
   }
 }
