@@ -45,10 +45,12 @@ bool IsProto3Field(const FieldDescriptor* field_descriptor) {
 }
 
 void SetMessageVariables(const FieldDescriptor* descriptor,
-                         map<string, string>* variables,
+                         std::map<string, string>* variables,
                          const Options& options) {
   SetCommonFieldVariables(descriptor, variables, options);
-  (*variables)["type"] = FieldMessageTypeName(descriptor);
+  (*variables)["type"] = ClassName(descriptor->message_type(), false);
+  (*variables)["file_namespace"] =
+      FileLevelNamespace(descriptor->file()->name());
   (*variables)["stream_writer"] =
       (*variables)["declared_type"] +
       (HasFastArraySerialization(descriptor->message_type()->file(), options)
@@ -137,7 +139,7 @@ GenerateAccessorDeclarations(io::Printer* printer) const {
 void MapFieldGenerator::
 GenerateInlineAccessorDefinitions(io::Printer* printer,
                                   bool is_inline) const {
-  map<string, string> variables(variables_);
+  std::map<string, string> variables(variables_);
   variables["inline"] = is_inline ? "inline" : "";
   printer->Print(variables,
       "$inline$ const ::google::protobuf::Map< $key_cpp$, $val_cpp$ >&\n"
@@ -154,7 +156,7 @@ GenerateInlineAccessorDefinitions(io::Printer* printer,
 
 void MapFieldGenerator::
 GenerateClearingCode(io::Printer* printer) const {
-  map<string, string> variables(variables_);
+  std::map<string, string> variables(variables_);
   variables["this_message"] = dependent_field_ ? DependentBaseDownCast() : "";
   printer->Print(variables, "$this_message$$name$_.Clear();\n");
 }
@@ -173,13 +175,19 @@ void MapFieldGenerator::
 GenerateConstructorCode(io::Printer* printer) const {
   if (HasDescriptorMethods(descriptor_->file(), options_)) {
     printer->Print(variables_,
-        "$name$_.SetAssignDescriptorCallback(\n"
-        "    protobuf_AssignDescriptorsOnce);\n"
-        "$name$_.SetEntryDescriptor(\n"
-        "    &$type$_descriptor_);\n");
+                   "$name$_.SetAssignDescriptorCallback(\n"
+                   "    $file_namespace$::protobuf_AssignDescriptorsOnce);\n"
+                   "$name$_.SetEntryDescriptor(\n"
+                   "    &$type$_descriptor);\n");
   }
 }
 
+void MapFieldGenerator::
+GenerateCopyConstructorCode(io::Printer* printer) const {
+  GenerateConstructorCode(printer);
+  GenerateMergingCode(printer);
+}
+
 void MapFieldGenerator::
 GenerateMergeFromCodedStream(io::Printer* printer) const {
     const FieldDescriptor* key_field =
@@ -224,7 +232,7 @@ GenerateMergeFromCodedStream(io::Printer* printer) const {
           "->AddLengthDelimited($number$, data);\n");
     } else {
       printer->Print(variables_,
-          "    unknown_fields_stream.WriteVarint32($tag$);\n"
+          "    unknown_fields_stream.WriteVarint32($tag$u);\n"
           "    unknown_fields_stream.WriteVarint32(data.size());\n"
           "    unknown_fields_stream.WriteString(data);\n");
     }
@@ -252,7 +260,7 @@ GenerateMergeFromCodedStream(io::Printer* printer) const {
 }
 
 static void GenerateSerializationLoop(io::Printer* printer,
-                                      const map<string, string>& variables,
+                                      const std::map<string, string>& variables,
                                       bool supports_arenas,
                                       const string& utf8_check,
                                       const string& loop_header,
@@ -291,17 +299,17 @@ static void GenerateSerializationLoop(io::Printer* printer,
 
 void MapFieldGenerator::
 GenerateSerializeWithCachedSizes(io::Printer* printer) const {
-  map<string, string> variables(variables_);
+  std::map<string, string> variables(variables_);
   variables["write_entry"] = "::google::protobuf::internal::WireFormatLite::Write" +
                              variables["stream_writer"] + "(\n            " +
                              variables["number"] + ", *entry, output)";
-  variables["deterministic"] = "output->IsSerializationDeterminstic()";
+  variables["deterministic"] = "output->IsSerializationDeterministic()";
   GenerateSerializeWithCachedSizes(printer, variables);
 }
 
 void MapFieldGenerator::
 GenerateSerializeWithCachedSizesToArray(io::Printer* printer) const {
-  map<string, string> variables(variables_);
+  std::map<string, string> variables(variables_);
   variables["write_entry"] =
       "target = ::google::protobuf::internal::WireFormatLite::\n"
       "                   InternalWrite" + variables["declared_type"] +
@@ -312,7 +320,7 @@ GenerateSerializeWithCachedSizesToArray(io::Printer* printer) const {
 }
 
 void MapFieldGenerator::GenerateSerializeWithCachedSizes(
-    io::Printer* printer, const map<string, string>& variables) const {
+    io::Printer* printer, const std::map<string, string>& variables) const {
   printer->Print(variables,
       "if (!this->$name$().empty()) {\n");
   printer->Indent();
@@ -399,7 +407,8 @@ void MapFieldGenerator::GenerateSerializeWithCachedSizes(
 void MapFieldGenerator::
 GenerateByteSize(io::Printer* printer) const {
   printer->Print(variables_,
-      "total_size += $tag_size$ * this->$name$_size();\n"
+      "total_size += $tag_size$ *\n"
+      "    ::google::protobuf::internal::FromIntSize(this->$name$_size());\n"
       "{\n"
       "  ::google::protobuf::scoped_ptr<$map_classname$> entry;\n"
       "  for (::google::protobuf::Map< $key_cpp$, $val_cpp$ >::const_iterator\n"
