@@ -165,7 +165,10 @@ DefaultValueObjectWriter* DefaultValueObjectWriter::RenderBytes(
   if (current_ == NULL) {
     ow_->RenderBytes(name, value);
   } else {
-    RenderDataPiece(name, DataPiece(value, false, true));
+    // Since StringPiece is essentially a pointer, takes a copy of "value" to
+    // avoid ownership issues.
+    string_values_.push_back(new string(value.ToString()));
+    RenderDataPiece(name, DataPiece(*string_values_.back(), false, true));
   }
   return this;
 }
@@ -187,7 +190,7 @@ void DefaultValueObjectWriter::RegisterFieldScrubCallBack(
 
 DefaultValueObjectWriter::Node::Node(
     const string& name, const google::protobuf::Type* type, NodeKind kind,
-    const DataPiece& data, bool is_placeholder, const vector<string>& path,
+    const DataPiece& data, bool is_placeholder, const std::vector<string>& path,
     bool suppress_empty_list, FieldScrubCallBack* field_scrub_callback)
     : name_(name),
       type_(type),
@@ -307,7 +310,7 @@ void DefaultValueObjectWriter::Node::PopulateChildren(
 
     // This code is checking if the field to be added to the tree should be
     // scrubbed or not by calling the field_scrub_callback_ callback function.
-    vector<string> path;
+    std::vector<string> path;
     if (!path_.empty()) {
       path.insert(path.begin(), path_.begin(), path_.end());
     }
@@ -463,7 +466,7 @@ DataPiece DefaultValueObjectWriter::CreateDefaultDataPieceForField(
 DefaultValueObjectWriter* DefaultValueObjectWriter::StartObject(
     StringPiece name) {
   if (current_ == NULL) {
-    vector<string> path;
+    std::vector<string> path;
     root_.reset(new Node(name.ToString(), &type_, OBJECT, DataPiece::NullData(),
                          false, path, suppress_empty_list_,
                          field_scrub_callback_.get()));
@@ -511,7 +514,7 @@ DefaultValueObjectWriter* DefaultValueObjectWriter::EndObject() {
 DefaultValueObjectWriter* DefaultValueObjectWriter::StartList(
     StringPiece name) {
   if (current_ == NULL) {
-    vector<string> path;
+    std::vector<string> path;
     root_.reset(new Node(name.ToString(), &type_, LIST, DataPiece::NullData(),
                          false, path, suppress_empty_list_,
                          field_scrub_callback_.get()));
@@ -554,26 +557,29 @@ DefaultValueObjectWriter* DefaultValueObjectWriter::EndList() {
 void DefaultValueObjectWriter::RenderDataPiece(StringPiece name,
                                                const DataPiece& data) {
   MaybePopulateChildrenOfAny(current_);
-  util::StatusOr<string> data_string = data.ToString();
   if (current_->type() != NULL && current_->type()->name() == kAnyType &&
-      name == "@type" && data_string.ok()) {
-    const string& string_value = data_string.ValueOrDie();
-    // If the type of current_ is "Any" and its "@type" field is being set here,
-    // sets the type of current_ to be the type specified by the "@type".
-    util::StatusOr<const google::protobuf::Type*> found_type =
-        typeinfo_->ResolveTypeUrl(string_value);
-    if (!found_type.ok()) {
-      GOOGLE_LOG(WARNING) << "Failed to resolve type '" << string_value << "'.";
-    } else {
-      current_->set_type(found_type.ValueOrDie());
-    }
-    current_->set_is_any(true);
-    // If the "@type" field is placed after other fields, we should populate
-    // other children of primitive type now. Otherwise, we should wait until the
-    // first value field is rendered before we populate the children, because
-    // the "value" field of a Any message could be omitted.
-    if (current_->number_of_children() > 1 && current_->type() != NULL) {
-      current_->PopulateChildren(typeinfo_);
+      name == "@type") {
+    util::StatusOr<string> data_string = data.ToString();
+    if (data_string.ok()) {
+      const string& string_value = data_string.ValueOrDie();
+      // If the type of current_ is "Any" and its "@type" field is being set
+      // here, sets the type of current_ to be the type specified by the
+      // "@type".
+      util::StatusOr<const google::protobuf::Type*> found_type =
+          typeinfo_->ResolveTypeUrl(string_value);
+      if (!found_type.ok()) {
+        GOOGLE_LOG(WARNING) << "Failed to resolve type '" << string_value << "'.";
+      } else {
+        current_->set_type(found_type.ValueOrDie());
+      }
+      current_->set_is_any(true);
+      // If the "@type" field is placed after other fields, we should populate
+      // other children of primitive type now. Otherwise, we should wait until
+      // the first value field is rendered before we populate the children,
+      // because the "value" field of a Any message could be omitted.
+      if (current_->number_of_children() > 1 && current_->type() != NULL) {
+        current_->PopulateChildren(typeinfo_);
+      }
     }
   }
   Node* child = current_->FindChild(name);
@@ -583,7 +589,6 @@ void DefaultValueObjectWriter::RenderDataPiece(StringPiece name,
         new Node(name.ToString(), NULL, PRIMITIVE, data, false,
                  child == NULL ? current_->path() : child->path(),
                  suppress_empty_list_, field_scrub_callback_.get()));
-    child = node.get();
     current_->AddChild(node.release());
   } else {
     child->set_data(data);
